#include "keyboard.h"

uint8_t mac[6] = {0};

void setup()
{
  //Load data
  load_config();
#ifdef DEBUG_K
  Serial.begin(115200);
  Serial.println();
  Serial.println("\n-----------------------");
  Serial.printf("K_mode: %d\n", Keyboard_Config.K_mode);
  Serial.printf("bluetooth_channel: %d\n", Keyboard_Config.Bluetooth_channel);
  Serial.printf("light_mode: %d\n", Keyboard_Config.L_mode);
  Serial.printf("brgightness: %d\n", Keyboard_Config.L_brgightness);
  Serial.printf("keyborad_system: %d\n", Keyboard_Config.Switch_system);
  Serial.printf("L_color: %d\n", Keyboard_Config.L_color);
  Serial.printf("L_speed: %d\n", Keyboard_Config.L_color);
  Serial.println("-----------------------");
  print_wakeup_reason();
  print_GPIO_wake_up();
#endif
  rtc_gpio_hold_dis(GPIO_NUM_1);
  rtc_gpio_hold_dis(GPIO_NUM_2);
  rtc_gpio_hold_dis(GPIO_NUM_3);
  rtc_gpio_hold_dis(GPIO_NUM_4);
  rtc_gpio_hold_dis(GPIO_NUM_5);
  rtc_gpio_hold_dis(GPIO_NUM_6);
  rtc_gpio_hold_dis(GPIO_NUM_7);
  rtc_gpio_hold_dis(GPIO_NUM_8);
  rtc_gpio_hold_dis(GPIO_NUM_9);
  rtc_gpio_hold_dis(GPIO_NUM_10);
  rtc_gpio_hold_dis(GPIO_NUM_11);
  rtc_gpio_hold_dis(GPIO_NUM_12);
  rtc_gpio_hold_dis(GPIO_NUM_13);
  rtc_gpio_hold_dis(GPIO_NUM_14);
  rtc_gpio_hold_dis(GPIO_NUM_15);
  rtc_gpio_hold_dis(GPIO_NUM_16);
  rtc_gpio_hold_dis(GPIO_NUM_17);
  rtc_gpio_hold_dis(GPIO_NUM_18);
  rtc_gpio_hold_dis(GPIO_NUM_21);

  pinMode(BAT_DET_CON, INPUT);
  pinMode(KEY_ENTER_PIN, INPUT_PULLDOWN);
  //Initialize row output pins and set default state
  for (int i = 0; i < NumRows; i++)
  {
    pinMode(Rows[i], OUTPUT);
    digitalWrite(Rows[i], LOW);
  }

  //Initialize column pins as input using pulldown (if available - some pins may require hardware pulldown)
  for (int i = 0; i < NumCols; i++)
  {
    pinMode(Cols[i], INPUT_PULLDOWN);
  }

  FastLED.addLeds<LedType, WS2812Pin, ColorOrder>(leds, WS2812_NUMBER);
  FastLED.setBrightness(Keyboard_Config.L_brgightness);
  fill_solid(leds, WS2812_NUMBER, CRGB::Black);
  FastLED.show();
  Wire.begin(SDA_PIN, SCL_PIN);
  Wire.setClock(400000);
  byte error, address;
  int nDevices = 0;
  Serial.println("Scanning for I2C devices ...");
  for (address = 0x01; address < 0x7f; address++) {
    Wire.beginTransmission(address);
    error = Wire.endTransmission();
    if (error == 0) {
      Serial.printf("I2C device found at address 0x%02X\n", address);
      nDevices++;
    } else if (error != 2) {
      Serial.printf("Error %d at address 0x%02X\n", error, address);
    }
  }

  if (nDevices == 0) {
    Serial.println("No I2C devices found");
  }
  delay(100);

  int deviceChose = Keyboard_Config.Bluetooth_channel;                     //Read selected address from storage //从存储器中读取所选地址
  Serial.print("deviceChose:");
  Serial.println(deviceChose);
#ifdef DEBUG_K
  Serial.print(MACAddress[deviceChose][0], HEX);
  Serial.print(" ");
  Serial.print(MACAddress[deviceChose][1], HEX);
  Serial.print(" ");
  Serial.print(MACAddress[deviceChose][2], HEX);
  Serial.print(" ");
  Serial.print(MACAddress[deviceChose][3], HEX);
  Serial.print(" ");
  Serial.print(MACAddress[deviceChose][4], HEX);
  Serial.print(" ");
  Serial.println(MACAddress[deviceChose][5], HEX);
#endif
  memcpy(mac, MACAddress[deviceChose], 6);

  esp_base_mac_addr_set(mac); //Set the MAC address according to the stored value using the universally administered MAC address.
  //If it is a random device address, check the highest two bits of the address.
  //If it is "11", it is a static random address.
  //If it is "00", it is a non-resolvable private address.
  //If it is "01", it is a resolvable private address.
  //esp_ble_gap_set_rand_addr(mac);  //Use a random device address.
  delay(1000);
  esp_read_mac(mac, ESP_MAC_BT);
#ifdef DEBUG_K
  Serial.print("BL channel MAC: ");
  Serial.print(mac[0], HEX);
  Serial.print(" ");
  Serial.print(mac[1], HEX);
  Serial.print(" ");
  Serial.print(mac[2], HEX);
  Serial.print(" ");
  Serial.print(mac[3], HEX);
  Serial.print(" ");
  Serial.print(mac[4], HEX);
  Serial.print(" ");
  Serial.println(mac[5], HEX);
#endif
  esp_ble_tx_power_set(ESP_BLE_PWR_TYPE_DEFAULT, ESP_PWR_LVL_P9);
  scan_task_begin();  //Start the key scanning task
  light_task_begin(); //Start the lighting task
  set_light_mode(Keyboard_Config.L_mode);  //Select the lighting mode
  set_scan_keyboard(0); //Stop the scanning
  //Only one task mode can be enabled at a time. Tasks cannot be enabled simultaneously.
  switch (Keyboard_Config.K_mode)
  {
    case USB_MODE:  //USB mode
#ifdef DEBUG_K
      Serial.println("USB");
#endif
      usb_mode_led();
      set_scan_keyboard(1); //Start the scanning
      usb_task_begin();        // Start the USB task
      break;
    case BLE_MODE:
#ifdef DEBUG_K           //Bluetooth mode
      Serial.println("BT");
#endif
      selector_channel_led(deviceChose);
      set_scan_keyboard(1); //Start the scanning
      bt_task_begin();      //Start the BLE task
      break;
    default:
      Keyboard_Config.K_mode = 1;
      save_config();
      //USB mode
#ifdef DEBUG_K
      Serial.println("Default_USB");
#endif
      usb_mode_led();
      set_scan_keyboard(1); //Start the scanning
      //Start the USB task
      usb_task_begin();
      break;
  }
#ifndef DEBUG_K
  vTaskDelete(NULL);
#endif
  set_scan_keyboard(1); //Start the scanning

#ifdef DEBUG_K
  Serial.println("Keyboard ready");
#endif

}

void loop()
{
#ifdef DEBUG_K
  static uint8_t keyb[KEYBOARD_ROW][KEYBOARD_COLUMN] = {OFF};
  static uint8_t keyboard_new[KEYBOARD_ROW][KEYBOARD_COLUMN] = {OFF};
  if (xQueuePeek(KEYBOARD_SATE_QUEUE, &keyboard_new, 10) == pdTRUE)
  {
    uint8_t n = 0;
    for (int i = 0; i < KEYBOARD_ROW; i++)
    {
      for (int j = 0; j < KEYBOARD_COLUMN; j++)
      {
        if (keyboard_new[i][j] != keyb[i][j])
        {
          n = 1;
          keyb[i][j] = keyboard_new[i][j];
        }
      }
    }
    if (n == 1)
    {
      for (int i = 0; i < KEYBOARD_ROW; i++)
      {
        for (int j = 0; j < KEYBOARD_COLUMN; j++)
        {
          Serial.printf("[%d]", keyboard_new[i][j]);
        }
        Serial.println();
      }
    }
  }
#endif
  bool pinState = digitalRead(BAT_DET_CON);
  if (pinState == LOW) {
    // Enter sleep mode if pins are at low level
    board_sleep();
  }
  vTaskDelay(5);
}
